"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Http2Client = void 0;
const http2_1 = require("http2");
const { HTTP_STATUS_INTERNAL_SERVER_ERROR, HTTP2_HEADER_AUTHORITY, HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_METHOD, HTTP2_HEADER_PATH, HTTP2_HEADER_SCHEME, HTTP2_HEADER_STATUS, } = http2_1.constants;
const HTTP2_METHOD_WITH_PAYLOAD = ['POST', 'PUT'];
class Http2Client {
    constructor(url, options) {
        this.url = url;
        this.options = options;
        this.session = http2_1.connect(url, options);
    }
    close() {
        this.session.close();
    }
    async request(requestOptions, nativeOptions) {
        return new Promise((resolve, reject) => {
            if (this.session.closed || this.session.destroyed) {
                this._reconnect();
            }
            let status = -1;
            let contentTypeHeader = "STRING" /* STRING */;
            const responseHeaders = {};
            const data = [];
            const requestHeaders = this._getRequestHeaders(requestOptions);
            const req = this.session.request(requestHeaders, nativeOptions);
            req.setEncoding('utf8');
            if (requestOptions.method &&
                HTTP2_METHOD_WITH_PAYLOAD.includes(requestOptions.method)) {
                const requestPayload = requestOptions.payload;
                if (requestPayload) {
                    switch (requestPayload.type) {
                        case "BUFFER" /* BUFFER */:
                            req.write(requestPayload.data);
                            break;
                        case "JSON" /* JSON */:
                            req.write(Buffer.from(JSON.stringify(requestPayload.data)));
                            break;
                        case "STRING" /* STRING */:
                            req.write(Buffer.from(requestPayload.data));
                            break;
                    }
                }
            }
            req.on('response', (headers) => {
                if (headers[':status']) {
                    status = headers[':status'];
                }
                for (const name in headers) {
                    if (headers[name] && name !== HTTP2_HEADER_STATUS) {
                        const header = headers[name]?.toString();
                        if (header) {
                            responseHeaders[name] = header;
                            if (name === HTTP2_HEADER_CONTENT_TYPE && header.includes('json')) {
                                contentTypeHeader = "JSON" /* JSON */;
                            }
                        }
                    }
                }
            });
            req.on('data', (chunk) => {
                data.push(chunk);
            });
            req.on('end', () => {
                req.end();
                resolve({
                    body: this._getResponseBody(data, contentTypeHeader),
                    headers: responseHeaders,
                    statusCode: this._getResponseStatus(status),
                });
            });
            req.on('error', (err) => {
                req.end();
                reject(err);
            });
        });
    }
    _getRequestHeaders(options) {
        if (options.method === 'OPTIONS') {
            return {
                ...(options.authority && {
                    [HTTP2_HEADER_AUTHORITY]: options.authority,
                }),
                [HTTP2_HEADER_METHOD]: 'OPTIONS',
                ...(options.scheme && {
                    [HTTP2_HEADER_SCHEME]: options.scheme,
                }),
            };
        }
        else {
            return {
                ...(options.authority && {
                    [HTTP2_HEADER_AUTHORITY]: options.authority,
                }),
                [HTTP2_HEADER_METHOD]: options.method || 'GET',
                [HTTP2_HEADER_PATH]: options.path || '/',
                [HTTP2_HEADER_SCHEME]: options.scheme || 'https',
            };
        }
    }
    _getResponseBody(data, responseType) {
        switch (responseType) {
            case "BUFFER" /* BUFFER */:
                return data.join().toString();
            case "JSON" /* JSON */:
                return JSON.parse(data.join());
            case "STRING" /* STRING */:
            default:
                return data.join();
        }
    }
    _getResponseStatus(status) {
        return status === -1 ? HTTP_STATUS_INTERNAL_SERVER_ERROR : status;
    }
    _reconnect() {
        this.session = http2_1.connect(this.url, this.options);
    }
}
exports.Http2Client = Http2Client;
//# sourceMappingURL=client.js.map